/*!
 * \file ah.h
 *
 * \author FTDI
 * \date 20120114
 *
 * Copyright  2011 Future Technology Devices International Limited
 * Company Confidential
 *
 * Project: FT12x
 * Module: LPC1114-FT12x Eval
 *
 * Rivision History:
 * 0.1 - initial version
 */


#ifndef __AL_H__
#define __AL_H__

/* LPC related includes */
#include "..//..//CMSISv2p00_LPC11xx//inc//LPC11xx.h"
#include "..//..//CMSISv2p00_LPC11xx//inc//core_cm0.h"
#include "..//..//LPC1114//ssp.h"

/* FT12x related includes */
#include "mainloop.h"
#include "chap_9.h"
#include "ftdi.h"

/* Standard C libraries */
#include<stdio.h>
#include<string.h>

#define ENABLE_HID_INTF					/* Enable the HID Keyboard interface*/
#define TURN_OFF_LEDS					/* Turn off LED blinking */
#define INFRA_DEBUG_ENABLE		1		/* Enable UART debug logging */
#define SIZE_DEBUG_BUFFER		200		/* Size of the debug ring buffer */
#define UART_BAUD_RATE			115200	/* UART baud */
#define EP_BULKOUT	4					/* Bulk out EP number */
#define EP_BULKIN	5					/* Bulk in   EP number */
#define SIZE_BUFFER1 2048				/* Size of loopback buffer */
#define ENDPOINT_INTERVAL	10			/* Endpoint interval */

/*Turn on this macro to change from the original f/w for the purpose of achieving higher throuput*/
#define OPTMISE_FOR_THROUPUT	1

/*Turn on this macro to process interrupts in main loop*/
//#define	PROCESS_ISR_IN_MAIN		1

#define DELAY_RD_WR	1		/* Data write hold time (For Optimization=O1 => 5=200KBps, 1=250KBps)*/

#define ENABLE_SW1_SW2		/* This macro will enable interrupt on SW0 & SW1 lines */

#define FT12X_COMMAND 1
#define FT12X_DATA 	0

#define SZ_TEMP_BUFFER	130

#ifdef INFRA_DEBUG_ENABLE
	extern int currentDebugLevel;
#endif

#define SSP_NUM			1

extern volatile unsigned char ctrl_packet_type;
extern volatile unsigned int fsm_state;
extern volatile unsigned long SizeToTransfer;
extern volatile unsigned long SizeTransfered;
extern volatile unsigned char ledState;


typedef unsigned char   uint8;
typedef unsigned short  uint16;
typedef unsigned long   uint32;
typedef unsigned long long uint64;

typedef signed char   int8;
typedef signed short  int16;
typedef signed long   int32;
typedef signed long long int64;

typedef unsigned char	bool;

#define VOS_HANDLE	unsigned int


extern EPPFLAGS bEPPflags;
extern volatile unsigned long isr_pending;
extern unsigned char  direction;
extern unsigned char tempBuffer[SZ_TEMP_BUFFER];
extern volatile unsigned int BulkInCompleted;
extern volatile unsigned int DisableBulkOut;

extern volatile unsigned long LastClockTicks;
extern volatile unsigned long ClockTicks;

extern unsigned char TimerExpiredForBulkIn;
extern unsigned char isSpi;
extern unsigned char EnhancedModeEnabled;
extern uint16 idProduct;
extern uint8 KbdDataAvailable;

extern void FT12x_main(void);
extern void read_write_register(void);
extern void enable(void);
extern void disable(void);
extern void FT12x_SystemHalt(unsigned int arg);
extern void fn_usb_isr(void);
extern void FT12x_ReadBuffer(bool A0, uint8 *buffer, uint32 size);
extern void FT12x_WriteBuffer(bool A0, uint8 *buffer, uint32 size);
extern void FT12x_Delay(unsigned long ticks);
extern void change_fsm_state(unsigned int next_state);
extern void Set_LED(uint8 ledNo, uint8 val);
extern void FT12x_ResetTimer(void);
extern void UARTSend(uint8_t *BufferPtr, uint32_t Length);




#endif /* FT120_AL_H_ */
