/*!
 * \file Chap_9.c
 *
 * \author FTDI
 * \date 20120114
 *
 * Copyright © 2011 Future Technology Devices International Limited
 * Company Confidential
 *
 * Project: FT12x
 * Module: LPC1114-FT12x Eval
 *
 * Rivision History:
 * 0.1 - initial version
 */


#include <stdio.h>
#include <string.h>
#include "..//include//ci.h"
#include "..//include//mainloop.h"
#include "..//include//usb100.h"
#include "..//include//chap_9.h"
#include "..//include//ftdi.h"
#include "..//include//al.h"


#define NUM_ENDPOINTS   2

#define CONFIG_DESCRIPTOR_LENGTH    sizeof(USB_CONFIGURATION_DESCRIPTOR) \
									+ sizeof(USB_INTERFACE_DESCRIPTOR) \
									+ (NUM_ENDPOINTS * sizeof(USB_ENDPOINT_DESCRIPTOR))

extern CONTROL_XFER ControlData;
extern IO_REQUEST idata ioRequest;
extern EPPFLAGS bEPPflags;

uint16 idProduct;


USB_DEVICE_DESCRIPTOR DeviceDescr =
{
	sizeof(USB_DEVICE_DESCRIPTOR),
    USB_DEVICE_DESCRIPTOR_TYPE,
	SWAP(0x0110),
	0xEF,	// bDeviceClass    (Misc)
	0x02,	// bDeviceSubClass (common)
	0x01,	// bDeviceProtocol (IAD)
	EP0_PACKET_SIZE, //should be 8 as per VNC2 fw,   //EP0 maxPktSz
	SWAP(0x0403), //VID - FTDI
	SWAP(0x601A), //PID - CDC:0x6018, HID:0x6019, CDC+HID:0x601A,
	SWAP(0x0000), //bcdDevice 
	1,//iManufacturer
	2,//iProduct
	3,//iSerialNumber
	1//bNumConfigurations
};

#ifdef ENABLE_HID_INTF
USB_HID_DSC	dsc_hid =
{
	/* HID class specific descriptor */
	9,			//bLength;       
	33,			//bDscType;      
	SWAP(0x0110),		//bcdHID;
    0x00,		//bCountryCode;   
	1,			//bNumDescriptors;       
	0x22, 		//bDescriptorType - REPORT
    SWAP(0x0041)//bDescriptorLength;
};
#endif

CFG01 ConfigDescr =
{
    /* Configuration Descriptor */
    sizeof(USB_CFG_DSC),    // Size of this descriptor in bytes
    DSC_CFG,                // CONFIGURATION descriptor type
    sizeof(ConfigDescr),    // Total length of data for this cfg
#ifdef ENABLE_HID_INTF
    3,//2,                      // Number of interfaces in this cfg
#else
	2,
#endif
    1,                     // Index value of this configuration
    0,                     // Configuration string index
    0xC0,                  // Attributes : self powered 
    //0xE0,				    // Attributes : self powered, remote wakeup
    0,                     // Max power consumption (2X mA)

#ifdef ENABLE_HID_INTF
	/* Interface Association Descriptor */
	sizeof(USB_INT_ASSOC_DEC),				// bLength
	0x0B,									// bDescriptorType = 11
	0x00,									// bFirstInterface
	0x02,									// bInterfaceCount
	0x02,									// bFunctionClass (Communication Class)
	0x02,									// bFunctionSubClass (Abstract Control Model)
	0x01,									// bFunctionProcotol (V.25ter, Common AT commands)
	0x00,									// iInterface
#endif
    
/* Interface Descriptor - CDC-ACM control interface */
    sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    DSC_INTF,               // INTERFACE descriptor type
    0,                      // Interface Number
    0,                      // Alternate Setting Number
    1,                      // Number of endpoints in this intf
    COMM_INTF,              // Class code
    ABSTRACT_CONTROL_MODEL, // Subclass code
    V25TER,                 // Protocol code
    0,                      // Interface string index

    /* CDC Class-Specific Descriptors */
    sizeof(USB_CDC_HEADER_FN_DSC),CS_INTERFACE,DSC_FN_HEADER,0x0110,
    sizeof(USB_CDC_ACM_FN_DSC),CS_INTERFACE,DSC_FN_ACM,0x02,
    sizeof(USB_CDC_UNION_FN_DSC),CS_INTERFACE,DSC_FN_UNION,CDC_COMM_INTF_ID,CDC_DATA_INTF_ID,
    sizeof(USB_CDC_CALL_MGT_FN_DSC),CS_INTERFACE,DSC_FN_CALL_MGT,0x00,CDC_DATA_INTF_ID,

    /* Endpoint Descriptor */
	sizeof(USB_EP_DSC),DSC_EP,_EP01_IN,_INT,CDC_INT_EP_SIZE,0x02,
    
/* Interface Descriptor - CDC-ACM data interface*/
    sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    DSC_INTF,               // INTERFACE descriptor type
    1,                      // Interface Number
    0,                      // Alternate Setting Number
    2,                      // Number of endpoints in this intf
    DATA_INTF,              // Class code
    0,                      // Subclass code
    NO_PROTOCOL,            // Protocol code
    0,                      // Interface string index
    
    /* Endpoint Descriptors */
    sizeof(USB_EP_DSC),DSC_EP,_EP02_OUT,_BULK,CDC_BULK_OUT_EP_SIZE,0x00,
    sizeof(USB_EP_DSC),DSC_EP,_EP02_IN,_BULK,CDC_BULK_IN_EP_SIZE,0x00
    
#ifdef ENABLE_HID_INTF
/* Interface Descriptor - HID Keyboard interface */
	,sizeof(USB_INTF_DSC),	// Size of this descriptor in bytes
	DSC_INTF,				// INTERFACE descriptor type
	2,						// Interface Number
	0,						// Alternate Setting Number
	1,						// Number of endpoints in this intf
	0x03,					// Class code - HID
	0x01,					// Subclass code - Boot interface
	0x01,					// Protocol code - Keyboard
	0,						// Interface string index

	/* HID class specific descriptor */
	9,			//bLength;       
	33,			//bDscType;      
	SWAP(0x0110),		//bcdHID;
    0x00,		//bCountryCode;   
	1,			//bNumDescriptors;       
	0x22, 		//bDescriptorType - REPORT
    SWAP(0x0041),//bDescriptorLength;

	/* EP descriptor - EP INT IN */ 
	sizeof(USB_EP_DSC),DSC_EP,_EP03_IN,_INT,CDC_INT_IN_EP_SIZE,13
#endif
};



uint8 ft232_str0_descriptor[4] = {
	0x04, 0x03, 0x09, 0x04
};

uint8 ft232_str1_descriptor[10] = {
	0x0a, 0x03, 0x46, 0x00, 0x54, 0x00, 0x44, 0x00, 0x49, 0x00
};


//FT12x Evaluation Platform
uint8 ft232_str2_descriptor[52] = {
	0x34, 0x03,
	0x46, 0x00, //F
	0x54, 0x00, //T
	0x31, 0x00, // 1
	0x32, 0x00, // 2
	0x58, 0x00, // X
	0x20, 0x00, //<space>
	0x45, 0x00, //E
	0x76, 0x00, //v
	0x61, 0x00, //a
	0x6C, 0x00, //l
	0x75, 0x00, //u
	0x61, 0x00, //a
	0x74, 0x00, //t
	0x69, 0x00, //i
	0x6F, 0x00, //o
	0x6E, 0x00, //n
	0x20, 0x00, //<space>
	0x50, 0x00, //P
	0x6C, 0x00, //l
	0x61, 0x00, //a
	0x74, 0x00, //t
	0x66, 0x00, //f
	0x6F, 0x00, //o
	0x72, 0x00, //r
	0x6D, 0x00, //m
};

//serial number
uint8 ft232_str3_descriptor[18] = {
	0x12, 0x03, 0x31, 0x00, 0x32, 0x00, 0x33, 0x00, 0x34, 0x00,
	0x35, 0x00, 0x36, 0x00, 0x37, 0x00, 0x38, 0x00
};

#ifdef ENABLE_HID_INTF
unsigned char report1_descriptor[65] = {
	0x05, 0x01, 0x09, 0x06, 0xA1, 0x01, 0x05, 0x07, 
	0x19, 0xE0, 0x29, 0xE7, 0x15, 0x00, 0x25, 0x01, 
	0x75, 0x01, 0x95, 0x08, 0x81, 0x02, 0x95, 0x01, 
	0x75, 0x08, 0x81, 0x01, 0x95, 0x03, 0x75, 0x01, 
	0x05, 0x08, 0x19, 0x01, 0x29, 0x03, 0x91, 0x02, 
	0x95, 0x05, 0x75, 0x01, 0x91, 0x01, 0x95, 0x06, 
	0x75, 0x08, 0x15, 0x00, 0x26, 0xFF, 0x00, 0x05, 
	0x07, 0x19, 0x00, 0x2A, 0xFF, 0x00, 0x81, 0x00, 
	0xC0
};
#endif

/*
   *************************************************************************
   // USB Protocol Layer
   *************************************************************************
*/

void reserved(void)
{
	DBG(MSG_DEBUG,"reserved\n\r");
	stall_ep0();
}

/*
   *************************************************************************
   // USB standard device requests
   *************************************************************************
*/

void get_status(void)
{
	unsigned char endp, txdat[2];
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;
	unsigned char c;
	
	if (bRecipient == USB_RECIPIENT_DEVICE) {
		if(bEPPflags.bits.remote_wakeup == 1)
			txdat[0] = 3;
		else
			txdat[0] = 1;
		txdat[1]=0;
		single_transmit(txdat, 2);
	} else if (bRecipient == USB_RECIPIENT_INTERFACE) {
		txdat[0]=0;
		txdat[1]=0;
		single_transmit(txdat, 2);
	} else if (bRecipient == USB_RECIPIENT_ENDPOINT) {
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			c = FT12x_SelectEndpoint(endp*2 + 1);	/* Control-in */
		else
			c = FT12x_SelectEndpoint(endp*2);	/* Control-out */
		if(c & FT12X_STALL)
			txdat[0] = 1;
		else
			txdat[0] = 0;
		txdat[1] = 0;
		single_transmit(txdat, 2);
	} else
	{
		stall_ep0();
	}
}

void clear_feature(void)
{
	unsigned char endp;
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;

//	DBG(MSG_DEBUG, "ENTER clear_feature\r\n");

	if (bRecipient == USB_RECIPIENT_DEVICE
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_REMOTE_WAKEUP) {
		DISABLE;
		bEPPflags.bits.remote_wakeup = 0;
		ENABLE;
		single_transmit(0, 0);
	}
	else if (bRecipient == USB_RECIPIENT_ENDPOINT
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_ENDPOINT_STALL) 
	{
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			/* clear TX stall for IN on EPn. */
			FT12x_SetEndpointStatus(endp*2 + 1, 0);
		else
			/* clear RX stall for OUT on EPn. */
			FT12x_SetEndpointStatus(endp*2, 0);
		single_transmit(0, 0);
	} else
		stall_ep0();
//	DBG(MSG_DEBUG, "EXIT clear_feature\r\n");
}

void set_feature(void)
{
	unsigned char endp;
	unsigned char bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;

//	DBG(MSG_DEBUG, "ENTER set_feature\r\n");
	if (bRecipient == USB_RECIPIENT_DEVICE
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_REMOTE_WAKEUP) {
		DISABLE;
		bEPPflags.bits.remote_wakeup = 1;
		ENABLE;
		single_transmit(0, 0);
	}
	else if (bRecipient == USB_RECIPIENT_ENDPOINT
		&& ControlData.DeviceRequest.wValue == USB_FEATURE_ENDPOINT_STALL) {
		endp = (unsigned char)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
		if (ControlData.DeviceRequest.wIndex & (unsigned char)USB_ENDPOINT_DIRECTION_MASK)
			/* clear TX stall for IN on EPn. */
			FT12x_SetEndpointStatus(endp*2 + 1, 1);
		else
			/* clear RX stall for OUT on EPn. */
			FT12x_SetEndpointStatus(endp*2, 1);
		single_transmit(0, 0);
	} else
		stall_ep0();
//	DBG(MSG_DEBUG, "EXIT set_feature\r\n");
}

void set_address(void)
{
	FT12x_SetAddressEnable((unsigned char)(ControlData.DeviceRequest.wValue &
		DEVICE_ADDRESS_MASK), 1);
	single_transmit(0, 0);
}

extern unsigned char  buffer1[SIZE_BUFFER1];
void get_descriptor(void)
{
	unsigned char bDescriptor = MSB(ControlData.DeviceRequest.wValue);
	unsigned char index = LSB(ControlData.DeviceRequest.wValue);
	unsigned char i=0;

	//DBG(MSG_DEBUG, "ENTER get_descriptor des=%u idx=%u\r\n",(unsigned)bDescriptor,(unsigned)index);

	if (bDescriptor == USB_DEVICE_DESCRIPTOR_TYPE) 
	{
		code_transmit((unsigned char code *)&DeviceDescr, sizeof(USB_DEVICE_DESCRIPTOR));
	} 
	else if (bDescriptor == USB_CONFIGURATION_DESCRIPTOR_TYPE) 
	{
		code_transmit((unsigned char code *)&ConfigDescr, sizeof(CFG01));
	} 
	else if(bDescriptor == USB_STRING_DESCRIPTOR_TYPE)
	{
		i=0;
		switch(index)
		{
			case 0:
				memcpy(&buffer1[0]+i,(unsigned char *)&ft232_str0_descriptor,sizeof(ft232_str0_descriptor));
				i+=sizeof(ft232_str0_descriptor);
				break;
			case 1:
				memcpy(&buffer1[0]+i,(unsigned char *)&ft232_str1_descriptor,sizeof(ft232_str1_descriptor));
				i+=sizeof(ft232_str1_descriptor);
				break;
			case 2:
				memcpy(&buffer1[0]+i,(unsigned char *)&ft232_str2_descriptor,sizeof(ft232_str2_descriptor));
				i+=sizeof(ft232_str2_descriptor);
				break;
			case 3:
				memcpy(&buffer1[0]+i,(unsigned char *)&ft232_str3_descriptor,sizeof(ft232_str3_descriptor));
				i+=sizeof(ft232_str3_descriptor);
				break;
				
			default:
				break;
		}
		code_transmit((unsigned char *)buffer1, i);
	}
#ifdef ENABLE_HID_INTF
	else if(bDescriptor == 0x22)
	{
		switch(index)
		{
			case 0:
				code_transmit((unsigned char code *)&report1_descriptor, 65);
				break;

			default:
				break;
		}
	}
	else if(bDescriptor == 0x21)//20120912_CV-HID-test
	{
		code_transmit((unsigned char code *)&dsc_hid, sizeof(dsc_hid));
	}
#endif
	else
	{
		TDBG(MSG_DEBUG, "get_descriptor des=%u idx=%u STALLING\r\n",(unsigned)bDescriptor,(unsigned)index);
		stall_ep0();
	}
//	DBG(MSG_DEBUG, "EXIT get_descriptor\r\n");
}

void get_configuration(void)
{
	unsigned char c = bEPPflags.bits.configuration;

	single_transmit(&c, 1);
}

void set_configuration(void)
{
	if (ControlData.DeviceRequest.wValue == 0) {
		/* put device in unconfigured state */
		single_transmit(0, 0);
		DISABLE;
		bEPPflags.bits.configuration = 0;
		ENABLE;
		init_unconfig();
	} else if (ControlData.DeviceRequest.wValue == 1) {
		/* Configure device */
		single_transmit(0, 0);

		init_unconfig();
		init_config();

		DISABLE;
		bEPPflags.bits.configuration = 1;
		ENABLE;
	} else
		stall_ep0();
}

void get_interface(void)
{
	unsigned char txdat = 0;        /* Only/Current interface = 0 */
	single_transmit(&txdat, 1);
}

void set_interface(void)
{
	//if (ControlData.DeviceRequest.wValue == 0 && ControlData.DeviceRequest.wIndex == 0)
	if (ControlData.DeviceRequest.wValue == 0)
		single_transmit(0, 0);
	else
		stall_ep0();
}

